#!/usr/bin/perl
#
#  Copyright (C) 2002  Stanislav Sinyagin
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

# $Id: Apache2Handler.pm,v 1.6 2005/07/22 11:29:05 ssinyagin Exp $
# Stanislav Sinyagin <ssinyagin@yahoo.com>

# Apache mod_perl handler. See http://perl.apache.org

use strict;

# but of course remove the apache2 stuff
BEGIN { require '/usr/share/torrus/conf_defaults/torrus-config.pl'; }

use Torrus::DB;
use Torrus::Log;
use Torrus::Renderer;
use Torrus::SiteConfig;
use Torrus::ACL;

use CGI;

my $q = new CGI;

if( $Torrus::Renderer::globalDebug )
{
   print STDERR $q->Vars;
}
    
    my @paramNames = $q->param;

    &Torrus::Log::setLevel('debug');

    my %options = ();
    foreach my $name ( @paramNames )
    {
        if( $name =~ /^[A-Z]/ and $name ne 'SESSION_ID' )
        {
            $options{'variables'}->{$name} = $q->param($name);
        }
    }
    
    my( $fname, $mimetype, $expires );
    
    # create the renderer instance
    if( not defined( $Torrus::ApacheHandler::renderer ) )
    {
        $Torrus::ApacheHandler::renderer = new Torrus::Renderer();
        if( not defined( $Torrus::ApacheHandler::renderer ) )
        {
            report_error($q, "Server error: could not load renderer.");
            exit 0;
        }
    }

    #my $tree = $q->self_url; #$r->uri();
    #$tree =~ s/^.*\/(.*)$/$1/;
    my $tree = $q->param("tree");
    
    if( not $fname )
    {
        if( not $tree or not Torrus::SiteConfig::treeExists( $tree ) )
        {
            ( $fname, $mimetype, $expires ) =
                $Torrus::ApacheHandler::renderer->
                renderTreeChooser( %options );
        }
        else
        {
            #if( $Torrus::ApacheHandler::authorizeUsers and
            #    not $options{'acl'}->hasPrivilege( $options{'uid'}, $tree,
            #                                       'DisplayTree' ) )
            #{
            #    return report_error($r, 'Permission denied');
            #}

            my $config_tree = new Torrus::ConfigTree( -TreeName => $tree );
            if( not defined($config_tree) )
            {
                report_error($q, 'Configuration is not ready');
                exit 0;
            }

            my $token = $q->param('token');
            if( not $token )
            {
                my $path = $q->param('path');
                $path = '/' unless $path;
                $token = $config_tree->token($path);
                if( not $token )
                {
                    report_error($q, 'Invalid path');
                    exit 0;
                }
            }
            elsif( $token !~ /^S/ and
                   not defined( $config_tree->path( $token ) ) )
            {
                report_error($q, 'Invalid token');
                exit 0;
            }

            my $view = $q->param('view');
            
            ( $fname, $mimetype, $expires ) =
                $Torrus::ApacheHandler::renderer->
                render( $config_tree, $token, $view, %options );
            
            undef $config_tree;
        }
    }

    if( defined( $options{'acl'} ) )
    {
        undef $options{'acl'};
    }

    if( defined($fname) )
    {
        if( not -e $fname )
        {
            report_error($q, 'No such file or directory: ' . $fname);
        }
        
        Debug("Render returned $fname $mimetype $expires");
        
        print $q->header(-type=>'text/html; charset=UTF-8',
                         -expires=>'+'.$expires.'s',
                         -refresh=>'+'.$expires.'s');

        open(FILE, $fname);
        while (<FILE>) {
            print ;
        }
        close(FILE);
    }
    else
    {
        report_error($q, "Renderer returned error.\n" .
                           "Probably wrong directory permissions or " .
                            "directory missing:\n" .
                     $Torrus::Global::cacheDir);            
        exit 0;
    }
    
    if( not $Torrus::Renderer::globalDebug )
    {
        &Torrus::Log::setLevel('warn');
    }
    
    exit 0;
#}


sub report_error
{
    my $q = shift;
    my $msg = shift;
    
    print $q->header(-type=>'text/html',
                    -status=>'401 Application Error');
    print('Error: ' . $msg);
}


#1;

# Local Variables:
# mode: perl
# indent-tabs-mode: nil
# perl-indent-level: 4
# End:
