#!/bin/sh
# Copyright Rene Mayrhofer, 2001
# This shell script is distributed under the terms of the GNU General Public License.

printhelp() {
  echo "Usage: $0 <cert file> <cert export password> <key length> <days> <country code> "
  echo "          <state> <locality> <org.> <org. unit> <common name> <email>"
  echo
  echo "key length:      the length of the RSA key that will be created"
  echo "days:            the certificate will stay valid this number of days"
  echo "                 the created certificate file, otherwise the name of "
  echo "                 the created certificate request"
  echo
  echo "The other fields will be placed in the certificate request."
}

if [ $# -ne 11 ]; then
    printhelp
    exit 1
fi

# generate a CA-signed cert
umask 077
newreq=`tempfile`
newkey=`tempfile`
newcert=`tempfile`
outcert="$1"
if ! ( echo -e "$5\n$6\n$7\n$8\n$9\n${10}\n${11}\n\n\n" | \
      	openssl req -new -outform PEM -newkey rsa:$3 \
              	-nodes -keyout "$newkey" -keyform PEM \
                       -out "$newreq" -days $4 -sha1 \
                       -config /etc/ssl/openssl.cnf >/dev/null 2>&1 ); then
        echo "Error generating certificate request !"
	rm "$newreq"
	rm "$newkey"
        exit 1
fi

if ! ( yes | openssl ca -policy policy_anything -out "$newcert" \
        -config /etc/ssl/openssl.cnf -infiles "$newreq" >/dev/null 2>&1 ); then
        echo "Error signing certificate request with CA !"
	rm "$newreq"
	rm "$newkey"
	rm "$newcert"
        exit 2
fi

if ! ( echo "$2" | \
	openssl pkcs12 -export -in "$newcert" -inkey "$newkey" \
	-certfile /etc/ssl/cacert.pem -out "$outcert" \
        -name "Gibraltar client IPSec certificate" -passout stdin ); then
	echo "Error generating PKCS#12 format certificate !"
        exit 3
fi

echo "CA subject line is:"
openssl x509 -in /etc/ssl/cacert.pem -noout -subject
echo "certificate subject line is:"
openssl x509 -in "$newcert" -noout -subject

rm "$newreq"
rm "$newkey"
rm "$newcert"

exit 0

