#! /usr/bin/perl -w
#
# 19.04.05
# v0.51 Peter Gastinger
# 
# This program is a simple tool the extended radius attributes for the Gibraltar Firewall
# with following functions:
# - showing ldap-radius-attributes for users identified by the uid
# - modifying (add,replace, delete) ldap-radius-attributes for users identified by the uid.
#
# There is no special interface, commandline arguments rulez ;-) All you
# have to do, is to configure the necessary config file.
# You can easily add services, just add it in the config file.
#
# Showing attributes
#   usage: ./gibRadAttr.pl username
#
# Add/Modify attributes
#   usage: ./gibRadAttr.pl username attribute1=value1 attribute2=value2 ....
#
# Example: ./gibUser.pl testuser attribute=value
#
#      Format in the LDAP DIT(Directory Information Tree):
#        dn: uid=username,ou=radius,dc=gibraltar,dc=local
#        uid: username
#        objectclass: radiusprofile
#        attribute: value
#
# Delete an attribute: 
#   If you wish to delete an attribute, just add it as command line value WITHOUT a value
# Example: ./gibUser.pl testuser attribute 
#
# --------------------------------------------------------------------------
# Copyright notice
# --------------------------------------------------------------------------
# Copyright: Peter Gastinger, Mar.2005
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL'.
# --------------------------------------------------------------------------
#

use strict;
use Net::LDAP;
#use Data::Dumper;
my($usage,$ldap,$mesg,$result,$dn,$i,$j,$key,$isExistingAttribute);  
my(%attrdata,%replacedata);
my(@tmp,@existingattributes,@deletedata);
our($basedn,$radiusdn,$rootdn,$rootpwd,$ldapserver,$ldapport);
our(@attributes);

#Configfile
require "/etc/gibUser.conf";

#PROGRAM
#usage
$usage = "Usage: \n";
$usage.= "add/modify\n$0 username attribute=value\n";
$usage.= "delete\n$0 username attribute\n";
$usage.= "--------\n";
$usage.= "There are a lot of attributes:\n";

for ($i=0;$i<@attributes;$i++) {
$usage.= "\t".$attributes[$i]."\n";
}

# correct commandline arguments 
if ( @ARGV < 1 ) {
	print $usage;
	exit;
}

# connect to LDAP
$ldap = Net::LDAP->new($ldapserver,port=>$ldapport) or die $@;
$ldap->bind($rootdn,password=>$rootpwd,version=>3) or die $@;
$dn="uid=".$ARGV[0].",".$radiusdn.",".$basedn;

# subroutine
if ( @ARGV == 1) {
	showattribute($dn);
}

if ( @ARGV > 1) {
	modattribute(@ARGV);
}

#unbind
$ldap->unbind();


sub addclass{
  my (@arguments)=@_;
  $mesg = $ldap->add($dn,
    attr=>["uid" =>$arguments[0],
    "cn" =>$arguments[0],
    "objectclass" => ["radiusprofile","account"]]);
  $mesg->code() && warn "Couldn't add class: ".$mesg->error;
}



# function showattributes - shows attributes
sub showattribute{
  my ($dn)=$dn;
  
  $mesg = $ldap->search(
                        base   => $dn,
                        scope  => 'base',
                        filter => 'objectclass=radiusprofile'
                      );
 
  $mesg->code() && die $mesg->error;
   foreach $result($mesg->all_entries) {
    print $result->dump();
  }
}


# function modattribute - modifies attributes 
sub modattribute{
  my (@arguments)=@_;
  print "#Modifying user ... ".$dn."\n";
  #addclass(@arguments);
 
#parse commandline arguments
  for ($i=1;$i<@arguments;$i++) {
  	$arguments[$i]=$arguments[$i];
    for ($j=0;$j<@attributes;$j++) {
    	if ($arguments[$i] =~ /^$attributes[$j]/i) {
    		@tmp = split(/=/,$arguments[$i]);
#    		print "tmp:".Dumper(@tmp);
    		if (exists $tmp[1]) {
    			$attrdata{$tmp[0]} = $tmp[1];
    		}else{
    		    push(@deletedata,$tmp[0]);
    		}
    	}
    }
  }
  
#OK
#	print "DATEN:".Dumper(@deletedata);


#look for existing attributes
  $mesg = $ldap->search(
                        base   => $dn,
                        scope  => 'base',
                        filter => 'objectclass=radiusprofile',
                        attrs => [@deletedata,keys %attrdata]
                      );
 
  $mesg->code() && die $mesg->error;

  $result=$mesg->entry;
  @existingattributes=$result->attributes;
  
# print "RESULT".Dumper($mesg->entries);


  
#  print "\nexisting1:".Dumper(@existingattributes);
 # print "\nattr:".Dumper(%attrdata);  
# print "\ndelt:".Dumper(@deletedata);  


# compare existing attributes with command line attributes
# save new attributes in the hash %attrdata
# save existing attributes in the hash %replacedata
  for ($i=0;$i<keys %attrdata;$i++) {
	for ($j=0;$j<@existingattributes;$j++) {
  	  $existingattributes[$j]=lc($existingattributes[$j]);
  	  if (exists $attrdata{$existingattributes[$j]}) {
        $replacedata{$existingattributes[$j]}=$attrdata{$existingattributes[$j]};
        delete($attrdata{$existingattributes[$j]});
  	  }
  	}
  }

# test valid values
#  foreach $key (keys %attrdata) {
 # 	if ($attrdata{$key} eq undef) {
#  		delete($attrdata{$key});
 # 	}
#  }

# test valid values
#  foreach $key (keys %replacedata) {
#  	if ($replacedata{$key} eq undef) {
#  		delete($replacedata{$key});
#  	}
#  }

#print "\n\ndelete:\n".Dumper(@existingattributes);
#print "\n\ndelete:\n".Dumper(@deletedata); 

# save attributes-to-delete in the array @deletedata
  for ($i=0;$i<@deletedata;$i++) {
  	$isExistingAttribute="FALSE";
	for ($j=0;$j<@existingattributes;$j++) {
      if (lc($deletedata[$i]) eq lc($existingattributes[$j])) {	
        $isExistingAttribute="TRUE";
      }
  	}
  	if ($isExistingAttribute eq "FALSE") {
  		splice(@deletedata,$i,1);
  		#delete $deletedata[$i];
  	}
  }  
   
#print "\n\nadd:\n".Dumper(%attrdata);    
#print "\n\nreplace:\n".Dumper(%replacedata); 
#print "\n\ndelete:\n".Dumper(@deletedata);   
    
# add and replace attributes
 $mesg = $ldap->modify($dn,changes => [
 		add => [ %attrdata ],
 		replace => [ %replacedata ]
 		]); 
 $mesg->code() && warn "Couldn't modify: ".$mesg->error;

# delete attributes
 $mesg = $ldap->modify($dn,delete => [ @deletedata ]);
 $mesg->code() && die "Couldn't delete: ".$mesg->error;
}

