#! /usr/bin/perl
#
# 17.09.07
# v0.70 Peter Gastinger
# 
# This program is a simple usermanagement tool for the Gibraltar Firewall
# with following functions:
# - show gibraltarUsers with their attributes identified by the uid
# - show all gibraltarUsers and their attributes 
# - show services for a user in a human readable format
# - show all users w/o their attributes
# - adding gibraltarUsers in the LDAP-DIT
# - delete gibraltarUsers
# - modify gibraltarUsers
#
# There is no special interface, commandline arguments rulez ;-) All you
# have to do, is to configure the necessary config file.
# You can easily add services, just add it in the config file.
#
# ATTENTION: For example, if you want to add the service samba (of course, 
#            there is no such service, its just an example), just add "samba"
#            to @validservices. By adding that, the attribute isSAMBAUser will
#            be added to the user in the LDAP DIT!!!! Therefor you have to
#			 extend the LDAP SCHEMA!!!!!!!!!!!!!!   
# 
# usage: ./gibUser.pl add username userpassword [vorname=vorname] [nachname=nachname] [mail=mail] [service={0|1}]
#        ./gibUser.pl show username 
#        ./gibUser.pl show
#		 ./gibUser.pl showuserlist
#		 ./gibUser.pl services username
#        ./gibUser.pl del username
#        ./gibUser.pl mod username [userPassword=newPassword] [vorname=vorname] [nachname=nachname] [mail=mail] [service={0|1}]
# 
# Example: ./gibUser.pl add testuser testpasswd vorname=Test nachname=User mail=test@user.at squid=1 postfix=0
#
#      Format in the LDAP DIT(Directory Information Tree):
#        dn: uid=username,ou=users,dc=gibraltar,dc=local
#        uid: username
#        userpassword: userpassword
#		 givenname: Test
#		 sn: User
#		 mail: test@user.at
#        objectclass: gibraltarUser
#        isSQUIDUser: TRUE
#        isRADIUSUser: FALSE
#        isPOSTFIXUser: FALSE
#
# --------------------------------------------------------------------------
# Copyright notice
# --------------------------------------------------------------------------
# Copyright: Peter Gastinger, Mar.2005
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
# On Debian GNU/Linux systems, the complete text of the GNU General
# Public License can be found in `/usr/share/common-licenses/GPL'.
# --------------------------------------------------------------------------
#

use strict;
use Net::LDAP;
use Data::Dumper;
my($usage,$usage1,$usage2,$ldap,$mesg,$result,$attr,$uattr,$dn,$rdn,$i,$j,$m);  
our($basedn,$userdn,$radiusdn,$rootdn,$rootpwd,$ldapserver,$ldapport);
our(@availableservices,@validservices,@servicegroupradius,@objectclasses,@reihenfolge); #array
our %userattributes;
my(@nasidentifiers,@removenasidentifiers);
my %userdata; #hash


#Configfile
require "/etc/gibUser.conf";

#PROGRAM
#usage
$usage = "Usage: $0 add username userpassword [vorname=vorname] [nachname=nachname] [mail=mail] ";
for ($i=0;$i<@availableservices;$i++) {
$usage1 .= "[".$availableservices[$i]."={0|1}] ";
$usage2 .= "--".lc($availableservices[$i])."=1\tpermit ".lc($availableservices[$i])." authentication\n";
$usage2 .= "--".lc($availableservices[$i])."=0\tdeny ".lc($availableservices[$i])." authentication\n";
}
$usage.=$usage1;
$usage.="\n$0 mod username [userpassword=newpasswd] ".$usage1;
$usage.="\n$0 {show|del} username";
$usage.="\n$0 show";
$usage.="\n$0 showuserlist";
$usage.="\n$0 services username";
$usage.="\n\nThis script adds,dels,modifies and shows users for the gibraltar Firewall";
$usage.="\n\n".$usage2;

# correct commandline arguments 
if ( (@ARGV == 0) || ($ARGV[0]=~/^add|mod$/ && @ARGV < 3) || ($ARGV[0]=~/^del$/ && @ARGV < 2) || ($ARGV[0]=~/^show$/ && @ARGV <1)) {
	print $usage;
	exit;
}

# connect to LDAP
$ldap = Net::LDAP->new($ldapserver,port=>$ldapport) or die $@;
$ldap->bind($rootdn,password=>$rootpwd,version=>3) or die $@;

if (@ARGV > 1) {
	$dn="uid=".$ARGV[1].",".$userdn.",".$basedn;
	$rdn="uid=".$ARGV[1].",".$radiusdn.",".$basedn;
}
# handle commandline arguments
if ($ARGV[0] =~ /^show$/ && @ARGV==1) {
  showuser("*");
}

if ($ARGV[0] =~ /^show$/) {
  showuser($ARGV[1]);
}

if ($ARGV[0] =~ /^services$/) {
  showservices(@ARGV);
}

if ($ARGV[0] =~ /^showuserlist$/) {
  showuserlist(@ARGV);
}

if ($ARGV[0] =~ /^add$/) {
  adduser(@ARGV);
}
if ($ARGV[0] =~ /^del$/) {
  deluser(@ARGV);
}
if ($ARGV[0] =~ /^mod$/) {
  moduser(@ARGV);
}

#unbind
$ldap->unbind();

# function showuser - shows an user identified by its uid
sub showuser{
  my (@arguments)=@_;
  $mesg = $ldap->search(base=>$userdn.",".$basedn,filter=>"(&(uid=$arguments[0]))");
  $mesg->code() && die $mesg->error;
  my @entries = $mesg->entries;
  foreach my $entr ( @entries ) {
   my $attr;
     foreach $attr ( sort $entr->attributes ) {
     # skip binary we can't handle
       if ( $attr =~ /userPKCS12/i ) {
       	print " $attr: binary \n";
       	next;
       }
       print "  $attr : ", $entr->get_value ( $attr ) ,"\n";
     }  	
     print "------------------------------\n";
  }
}

# function showuserlist - shows all users with only their usernames
sub showuserlist{
  my @entries;
  my $entr;
# LDAP search attributes in the $userdn
  $mesg = $ldap->search(base=>$userdn.",".$basedn,filter=>"(&(uid=*))",attrs=>["uid"]);
  $mesg->code() && die $mesg->error;
  @entries = $mesg->entries;
  foreach $entr (sort @entries ) {  
	print $entr->get_value ( "uid" )."\n";
  }  
}

# function showservices - shows services of an user identified by its uid in
# human understandable format
sub showservices{
  my (@arguments)=@_;
  my @entries;
  my($entr,$attr);
  my %servicesout;

# LDAP search attributes in the $userdn
  $mesg = $ldap->search(base=>$userdn.",".$basedn,filter=>"(&(uid=$arguments[1]))");
  $mesg->code() && die $mesg->error;
  
  
# save entries in a hash, mapping between ldap entries and human readable entries
 @entries = $mesg->entries;
   foreach $entr ( @entries ) {
     foreach $attr (sort $entr->attributes ) {
		foreach $uattr (keys %userattributes) {
        if (lc($attr) =~ /^$uattr$/) {
			$servicesout{$userattributes{$uattr}}=$entr->get_value ( $attr );		
	   	  }     							
		}
		for ($i=0;$i<@validservices;$i++) {
      	  if (lc($attr) =~ /$validservices[$i]/ && $entr->get_value($attr)=~"TRUE") {
			$servicesout{$validservices[$i]}=$validservices[$i];		
    	  }     				
		}
     }
   }
   
# LDAP search attributes in the $radiusdn  
  $mesg = $ldap->search(base=>$radiusdn.",".$basedn,filter=>"(&(uid=$arguments[1]))",attrs=>['radiusNASIdentifier']);
  $mesg->code() && die $mesg->error;  
  
 @entries = $mesg->entries;
   foreach $entr ( @entries ) {
     foreach $attr (sort $entr->attributes ) {
		for ($i=0;$i<@servicegroupradius;$i++) {
			my @tmp=$entr->get_value($attr);
			for ($j=0;$j<@tmp;$j++) {
      	      if (lc($tmp[$j]) =~ /$servicegroupradius[$i]/) {
			  $servicesout{$servicegroupradius[$i]}=$servicegroupradius[$i];
      	      }
    	    }     				
		}
     }
   } 
  
# print to stdout in the given order (gibUser.conf)   
   for ($i=0;$i<@reihenfolge;$i++) {
   	if (exists $servicesout{$reihenfolge[$i]}) {   	
   	  print $reihenfolge[$i].": ".$servicesout{$reihenfolge[$i]}."\n";
   	}
   }
   for ($i=0;$i<@availableservices;$i++) {
   	if (exists $servicesout{$availableservices[$i]}) {
   		print $servicesout{$availableservices[$i]}."\n";
   	}
   } 
}



# function adduser - adds an user 
sub adduser{
  my (@arguments)=@_;
  print "#Adding user ... ".$arguments[1]."\n";

  $userdata{"uid"} =  $arguments[1];
  $userdata{"userpassword"} = $arguments[2];


#default - every service denied
  for ($i=0;$i<@validservices;$i++) {
  		$userdata{"is".uc($validservices[$i])."User"}="FALSE";
  }
  
  $userdata{"radiusprofiledn"}="uid=".$arguments[1].",".$radiusdn.",".$basedn;
  if (@arguments > 2) {
    for ($i=2;$i<@arguments;$i++) {
    	
      if ($arguments[$i] =~ /vorname=(.*)/) {
        $userdata{"givenname"}=$1;
      }	
      if ($arguments[$i] =~ /nachname=(.*)/) {
        $userdata{"sn"}=$1;
      }	
      if ($arguments[$i] =~ /mail=(.*)/) {
        $userdata{"mail"}=$1;
      }
	      
      for ($j=0;$j<@validservices;$j++) {
        if ($arguments[$i] =~ /($validservices[$j])=([1|0]{1})/) {
	      $userdata{"is".uc($validservices[$j])."User"}=(($2==1)?"TRUE":"FALSE");					
        }
      }	
      
      for ($j=0;$j<@servicegroupradius;$j++) {
        if ($arguments[$i] =~ /($servicegroupradius[$j])=1/) {
	      $userdata{"isRADIUSUser"}="TRUE";
	      push(@nasidentifiers, $1);	
        }
      }	      
    }    	
  }
    
#generate LDIF format output
  foreach $attr (keys %userdata) {
    print "$attr: $userdata{$attr}\n";
  }
  foreach $attr (@objectclasses) {
    print "objectclass: $attr\n";
  }

#add user 
  $mesg = $ldap->add($dn,
    attr=>[%userdata,
    objectclass => [@objectclasses]]);
 $mesg->code() && warn "Couldn't add record: ".$mesg->error;

#add radiusattributes 
if (@nasidentifiers!=0) {
   $mesg = $ldap->add("uid=".$arguments[1].",".$radiusdn.",".$basedn,
    attr=>["uid" =>$arguments[1],
    "cn" =>$arguments[1],
    "radiusNASIdentifier" => [@nasidentifiers],
    "objectclass" => ["radiusprofile","account"]]);	
} else  {
   $mesg = $ldap->add("uid=".$arguments[1].",".$radiusdn.",".$basedn,
    attr=>["uid" =>$arguments[1],
    "cn" =>$arguments[1],
    "objectclass" => ["radiusprofile","account"]]);	
}
  $mesg->code() && warn "Couldn't add record: ".$mesg->error;
   
}


# function deluser - delete an user 
sub deluser{
  my (@arguments)=@_;
  print "#Deleting user ... ".$arguments[1]."\n";
  $mesg = $ldap->delete($dn);
  $mesg->code() && warn "Couldn't delete: ".$mesg->error;
  $mesg = $ldap->delete("uid=".$arguments[1].",".$radiusdn.",".$basedn);
  $mesg->code() && warn "Couldn't delete: ".$mesg->error;  
}


# function moduser - modifies an user 
# separating chillispot and vpn is more complex then the other services, so
# this function is also much more complex! :-)
# If there is a commandline argument chilli or vpn (or something defined in gibUser.conf in
# the @servicegroupradius-array), then there must be an entry in the ldap, in the ou=radius,
# named radiusnasidentifier with value chilli or vpn or both (multivalued value). Additionally
# isRadiusUser-Access-Attribut (if you want to use a radius service) must be set to 
# "TRUE", otherwise it doesn't work!!!
#
sub moduser{
 my (@arguments)=@_;
 my (@changeableattributes)=("userpassword","vorname","nachname","mail");
 my (@changeableattributes_map)=("userpassword","givenname","sn","mail");
 my $itemexist;
 my($entr,$attr);
 my @tmp;

  print "#Modifying user ... ".$arguments[1]."\n";
 
  $userdata{"uid"} =  $arguments[1];

  $mesg = $ldap->search(base=>$radiusdn.",".$basedn,filter=>"(&(uid=$arguments[1]))",attrs=>['radiusNASIdentifier']);
  $mesg->code() && die $mesg->error;  
  
  @tmp = $mesg->entries;
  $entr=$tmp[0];
  @tmp = $entr->get_value("radiusNASIdentifier");

  if (@arguments > 1) {
    for ($i=2;$i<@arguments;$i++) {
      for ($m=0;$m<@changeableattributes;$m++) {
  	  	if ($arguments[$i] =~ /$changeableattributes[$m]=(.*)/) {
  	      $userdata{$changeableattributes_map[$m]} = $1;
  	    }
      }    	      	
      for ($j=0;$j<@validservices;$j++) {
        if ($arguments[$i] =~ /$validservices[$j]=([1|0]{1})/) {
          $userdata{"is".uc($validservices[$j])."User"}=(($1==1)?"TRUE":"FALSE");
        }
      }      
      for ($j=0;$j<@servicegroupradius;$j++) {
        if ($arguments[$i] =~ /($servicegroupradius[$j])=1/) {
	      push(@nasidentifiers, $1);	
        }
        if ($arguments[$i] =~ /($servicegroupradius[$j])=0/) {
	      push(@removenasidentifiers, $1);	
        }        
      }	      	
    }    	
  }
  
  for ($i=0;$i<@tmp;$i++) {
  	for ($j=0;$j<@removenasidentifiers;$j++) {
  		if ($tmp[$i] =~ /$removenasidentifiers[$j]/) {	
			delete $tmp[$i];  			
  		}	
  	}
  }
    
  for ($i=0;$i<@tmp;$i++) {
  	$itemexist=0;
  	if (exists $tmp[$i]) {
  		for ($j=0;$j<@nasidentifiers;$j++) {
  			if ($nasidentifiers[$j]=~$tmp[$i]) {
  				$itemexist=1;
  			}
  		}
  		if ($itemexist != 1) {
  			push(@nasidentifiers,$tmp[$i]);	
  		}
  	}	
  }
  
 if (@nasidentifiers>0) {
   $userdata{"isRADIUSUser"}="TRUE";
 }
 else {
 	$userdata{"isRADIUSUser"}="FALSE";
 }
 
 $mesg = $ldap->modify($dn,
 		replace => { %userdata } 
 		);
 $mesg->code() && warn "Couldn't modify: ".$mesg->error;

 $mesg = $ldap->modify($rdn,
 		replace => { radiusNASIdentifier => [@nasidentifiers] } 
 		);

}

