#!/bin/sh

# This script is executed when the service torrus-common does not start correctly
# after a power failure or something like that. There are two different states of
# recreating the databases and a special extension to remove the RRDs of the interfaces
# because of an extension within the database.
# The first one only removes the BDBs used by torrus; the
# second one also removes the RRDs containing the recorded data.
# RL - 2007-08-16
# RL - 2007-08-22 - added part to delete only the interfaces RRDs

BDB_PATH="/var/lib/torrus/db/";
RRD_PATH="/var/lib/torrus/collector_rrd/";
PID_FILE="/var/run/torrus/collector.main.pid";
IFACE_PATH="/var/lib/torrus/collector_rrd/ifaces/";

if [ $# -ne 1 ]; then
        echo "Usage: $0 <level of recreation> - 0 will only delete the BDBs; 1 will also delete the RRDs; 2 removes the interfaces RRDs.";
        exit 1;
fi

remove_bdbs()
{
        find $BDB_PATH -xtype f | xargs rm -f;
        rm -f $PID_FILE
}

remove_rrds()
{
        find $RRD_PATH -name "*rrd" | xargs rm -f;
}

remove_iface_rrds()
{
		find $IFACE_PATH -name "*rrd" | xargs rm -f;
}

# stopping the torrus service
/etc/init.d/torrus-common stop >/dev/null

if [ "$1" = "1" ]; then
        remove_rrds;
elif [ "$1" = "2" ]; then
		remove_iface_rrds;
else
        remove_bdbs;
fi

# compile xml file again
torrus compilexml --all
