#! /usr/bin/perl
#
# script to migrate:
#  pptp users (/etc/ppp/chap-secrets)
#  squid users (/etc/squid/user)
#  postfix users (/etc/sasldb2)
#

use strict;
use Data::Dumper;
use Net::LDAP;
use Sys::Hostname;

require "/etc/gibUser.conf";

my @users;
my %pptpusers;
my %squidusers;
my %postfixusers;
my $logfile = "/etc/gibraltar/usermigration.csv";

my $host = hostname;
print "migrating users from chap-secrets ...\n";
open(PPTP,"/etc/ppp/chap-secrets") or die "Unable to open chap-secrets file";;
while (<PPTP>){
#example entry: peda	gibraltar	"secret"	*
  if ($_ =~ /^(\w*)\s*($host|gibraltar-l2tp)\s*"(.*)"/){
    $pptpusers{$1} = $3;
  }
}
close(PPTP);

print "migrating squid users ... \n";
open(SQUID,"/etc/squid/users") or die "Unable to open squid users file";
while (<SQUID>){
#example entry: peda:q/5lzJZjhBF1c
  if ($_ =~ /^(\w*):*/){
    $squidusers{$1} = "";
  }
}
close(SQUID);

print "migrating postfix user ...\n";
open(POSTFIX,"/usr/sbin/sasldblistusers2 |");
while (<POSTFIX>) {
#example entry: peda@gibraltar.domain.tld: userPassword
  if ($_ =~ /^(\w*)@.*: userPassword$/){
    $postfixusers{$1} = "";
  }
}

@users = keys %squidusers;
foreach (keys %pptpusers) {
 if (!defined($squidusers{$_})) {
   push(@users,$_);
 }
}
foreach (keys %postfixusers) {
 if (!defined($squidusers{$_}) && !defined($pptpusers{$_})) {
   push(@users,$_);
 }
}

#print Dumper(@users);
#

my $runningslapd = 0;
# start ldap server and wait 2 seconds
if (system("/etc/init.d/slapd status") != 0) { 
  system("/etc/init.d/slapd start");
  sleep(2);
  $runningslapd = 1;
}

## PREPARING MIGRATION FILE ##
open(LOG,">$logfile");
print LOG "Gibraltar User Migration Report\n";
my $count = 0;
my $now = localtime time;
print LOG "generated on $now\n\n";
print LOG "----------------------------------------------\n";
print LOG "User;Password;Password_new;HTTP Proxy;VPN;Mail\n";

########## ADDING users
#
foreach (@users) {
  my($username,$password,$squid,$pptp,$postfix);
  $username=$_;
  if (defined($pptpusers{$username})) {
    $password=$pptpusers{$username};
    $pptp = 1;
  } else {
    $password=randomPassword();
    $pptp = 0;
  }

  if (defined($squidusers{$username})) {
    $squid = 1;
  } else {
    $squid = 0;
  }

  if (defined($postfixusers{$username})) {
    $postfix = 1;
  } else {
    $postfix = 0;
  }
$count++;
#print LOG "User: ".$count."\n";
my $new = 0;
if ($pptp == 0) {
 $new = 1;
}
print LOG "$username;$password;$new;$squid;$pptp;$postfix\n";
#if ($pptp == 0) {
#  print LOG "Generated new password this user\n";
# }
#print LOG "\n";
#print "add $username $password vpn=$pptp squid=$squid postfix=$postfix";
adduser("add","$username","$password","vpn=$pptp","squid=$squid","postfix=$postfix");
}

print LOG "----------------------------------------------\n";
print LOG "Migration process done - migrated $count users\n";
close(LOG);

# stop ldap server
if ($runningslapd == 1) {
  system("/etc/init.d/slapd stop");
}

sub randomPassword {
  my @password;
  my @numbers = split(" ","1 2 3 4 5 6 7 8 9 0");
  my @special = split(" ",", . : - _ + ? =");
  my @lowerchars = split(" ","a b c d e f g h i j k l m n o p q r s t u v w x y z");
  my @upperchars = split(" ","A B C D E F G H I J K L M N O P Q R S T U V W X Y Z");
  my @chars = (@lowerchars,@upperchars);
  my $password_length = $_[0];
  if (!$password_length || $password_length < 8) {
     $password_length = 8;
  }

  my $countspecialchars = int($password_length/4);
  my $countnumbers = int($password_length/4);
  my $countchars = $password_length - $countspecialchars - $countnumbers;

  srand;
  for (my $i=0; $i < $countspecialchars  ;$i++) {
    @password[$i] = $special[int(rand @special)];
  }
  for (my $i=0; $i < $countnumbers  ;$i++) {
    $password[$countspecialchars+$i] = $numbers[int(rand @numbers)];
  }
  for (my $i=0; $i < $countchars ;$i++) {
    $password[$password_length-$countchars+$i] = $chars[int (rand @chars)];
 }

  fisher_yates_shuffle( \@password );
  return join("",@password);
}


# function adduser - adds an user 
sub adduser{
my($usage,$usage1,$usage2,$ldap,$mesg,$result,$attr,$uattr,$dn,$rdn,$i,$j,$m);
our($basedn,$userdn,$radiusdn,$rootdn,$rootpwd,$ldapserver,$ldapport);
our %userattributes;
our(@availableservices,@validservices,@servicegroupradius,@objectclasses,@reihenfolge);
our %userattributes;
my(@nasidentifiers,@removenasidentifiers);
my %userdata; #hash


# connect to LDAP
$ldap = Net::LDAP->new($ldapserver,port=>$ldapport) or die $@;
$ldap->bind($rootdn,password=>$rootpwd,version=>3) or die $@;
  my (@arguments)=@_;
  $dn="uid=".$arguments[1].",".$userdn.",".$basedn;
  $rdn="uid=".$arguments[1].",".$radiusdn.",".$basedn;
#  print "#Adding user ... ".$arguments[1]."\n";

  $userdata{"uid"} =  $arguments[1];
  $userdata{"userpassword"} = $arguments[2];

#default - every service denied
  for ($i=0;$i<@validservices;$i++) {
  		$userdata{"is".uc($validservices[$i])."User"}="FALSE";
  }
  
  $userdata{"radiusprofiledn"}="uid=".$arguments[1].",".$radiusdn.",".$basedn;
  if (@arguments > 2) {
    for ($i=2;$i<@arguments;$i++) {
    	
      if ($arguments[$i] =~ /vorname=(.*)/) {
        $userdata{"givenname"}=$1;
      }	
      if ($arguments[$i] =~ /nachname=(.*)/) {
        $userdata{"sn"}=$1;
      }	
      if ($arguments[$i] =~ /mail=(.*)/) {
        $userdata{"mail"}=$1;
      }
	      
      for ($j=0;$j<@validservices;$j++) {
        if ($arguments[$i] =~ /($validservices[$j])=([1|0]{1})/) {
	      $userdata{"is".uc($validservices[$j])."User"}=(($2==1)?"TRUE":"FALSE");					
        }
      }	
      
      for ($j=0;$j<@servicegroupradius;$j++) {
        if ($arguments[$i] =~ /($servicegroupradius[$j])=1/) {
	      $userdata{"isRADIUSUser"}="TRUE";
	      push(@nasidentifiers, $1);	
        }
      }	      
    }    	
  }
    
#generate LDIF format output
#  foreach $attr (keys %userdata) {
#    print "$attr: $userdata{$attr}\n";
#  }
#  foreach $attr (@objectclasses) {
#    print "objectclass: $attr\n";
#  }

#print Dumper(%userdata);
#print "DN: $dn\n";
#add user 
  $mesg = $ldap->add($dn,
    attr=>[%userdata,
    objectclass => [@objectclasses]]);
 $mesg->code() && warn "Couldn't add record: ".$mesg->error;

#add radiusattributes 
if (@nasidentifiers!=0) {
   $mesg = $ldap->add("uid=".$arguments[1].",".$radiusdn.",".$basedn,
    attr=>["uid" =>$arguments[1],
    "cn" =>$arguments[1],
    "radiusNASIdentifier" => [@nasidentifiers],
    "objectclass" => ["radiusprofile","account"]]);	
} else  {
   $mesg = $ldap->add("uid=".$arguments[1].",".$radiusdn.",".$basedn,
    attr=>["uid" =>$arguments[1],
    "cn" =>$arguments[1],
    "objectclass" => ["radiusprofile","account"]]);	
}
  $mesg->code() && warn "Couldn't add record: ".$mesg->error;
   
$ldap->unbind();
}

#shuffle password array
sub fisher_yates_shuffle {
    my $array = shift;
    my $i;
    for ($i = @$array; --$i; ) {
      my $j = int rand ($i+1);
      next if $i == $j;
      @$array[$i,$j] = @$array[$j,$i];
    }
}
